//
// CellAutomataTableModel.java
//

import javax.swing.table.AbstractTableModel;

class CellAutomataTableModel extends AbstractTableModel 
{
    // OVERVIEW: Table model for representing a cellular automata grid.

    private int rows;
    private int columns;
    private Grid grid;

    CellAutomataTableModel (String cellClassName, int p_rows, int p_columns)
    {
        try {
            Class cellClass = Class.forName (cellClassName);
            rows = p_rows;
            columns = p_columns;
            grid = new Grid (cellClass, this);
        } catch  (ClassNotFoundException e) {
            System.out.println ("Could not find cell class.");
            System.exit (0);
        }
    }
    
    public int getColumns () 
    //@ensures \result >= 0;
    {
        return columns;
    }
    
    public int getRows () 
    //@ensures \result >= 0;
    {
        return rows;
    }
    
    // Need to implement these also
    public int getColumnCount () {
        return getColumns ();
    }
    public int getRowCount () {
        return getRows ();
    }

    public Object getValueAt (int row, int column) {
        return grid.get (row, column);
    }
    
    public String getColumnName (int column) {
        return "";
    }
    
    public void setValueAt (Object value, int row, int column) {
        if (value instanceof Boolean) {
            grid.setobject (row, column, (Boolean) value); 
        }
    }
    
    public Class getColumnClass (int column) {
        return Boolean.class;
    }
    
    public boolean isCellEditable (int row, int column) {
        return true;
    }

    public void step () {
        grid.step ();
    }
}
